#
# Copyright (C) 2018-2023  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.

"""Application-wide constants."""
from enum import Enum, unique

from backports.strenum import StrEnum

# Number of decimal places of currency amounts.
# Bitcoin has 8, so 10 should be enough for most practical purposes.
CURRENCY_PRECISION = 10


@unique
class PaymentType(StrEnum):
    """Payment types constants."""

    TRANSFER = 'transfer'
    CARD_PAYMENT = 'card_payment'


@unique
class PaymentState(StrEnum):
    """Payment states constants."""

    INITIALIZED = 'initialized'
    READY_TO_PROCESS = 'ready_to_process'
    PROCESSED = 'processed'
    DEFERRED = 'deferred'
    EXPORTED = 'exported'
    CANCELED = 'canceled'
    TO_REFUND = 'to_refund'
    REFUNDED = 'refunded'


@unique
class InvoiceType(str, Enum):
    """Invoice type constants."""

    ADVANCE = 'advance'
    ACCOUNT = 'account'


@unique
class ProcessPaymentResultType(Enum):
    """Result type of payment processing."""

    NOT_PROCESSED = 0  # evaluates to `False` in boolean context
    TO_REFUND = 1
    PROCESSED = 2


@unique
class PaymentProcessingError(str, Enum):
    """Payment processing error constants."""

    DUPLICITY = 'duplicity'
    INSUFFICIENT_AMOUNT = 'insufficient_amount'
    EXCESSIVE_AMOUNT = 'excessive_amount'
    OVERDUE = 'overdue'
    MANUALLY_BROKEN = 'manually_broken'
    TOO_OLD = 'too_old'


@unique
class MessageReference(str, Enum):
    """Type of reference when sending messages."""

    PAYMENT = 'payment'


@unique
class MessageTemplate(str, Enum):
    """Enum of templates."""

    CSOB_CUSTOMER_CODE_SUBJECT = 'csob-customer-code-subject.txt'
    CSOB_CUSTOMER_CODE_BODY = 'csob-customer-code-body.txt'
